/*!

  \file GateCoincidenceDigi.hh
  
  $Log: GateCoincidenceDigi.hh,v $
  Revision 1.5  2004/09/16 14:00:21  gsantin
  FileSizeLimit and Coinc/Single masks

  Revision 1.4  2002/08/11 15:33:24  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateCoincidenceDigi
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateCoincidenceDigi.hh,v 1.5 2004/09/16 14:00:21 gsantin Exp $
*/

#ifndef GateCoincidenceDigi_h
#define GateCoincidenceDigi_h 1

#include "G4VDigi.hh"
#include "G4TDigiCollection.hh"
#include "G4Allocator.hh"
#include "G4ThreeVector.hh"


#include "GateCoincidencePulse.hh"


class GateCoincidenceDigi : public G4VDigi
{

public:
  
  GateCoincidenceDigi();
  GateCoincidenceDigi(GateCoincidencePulse* coincidencePulse);
  GateCoincidenceDigi(const GateCoincidencePulse& coincidencePulse);
  virtual inline ~GateCoincidenceDigi() {}
  
  inline void* operator new(size_t);
  inline void  operator delete(void*);
  
  void Draw();
  void Print();

  //
  //printing methods
  //
  friend G4std::ostream& operator<<(G4std::ostream&, GateCoincidenceDigi&);    

  friend G4std::ofstream& operator<<(G4std::ofstream&, GateCoincidenceDigi*);    

public:
  
      inline GatePulse& GetPulse(G4int i)                     { return pulseVector[i]; }
      inline void SetPulse(G4int i, const GatePulse& value)   { pulseVector[i] = value; }

private:

      GatePulse pulseVector[2];

public:
  static void SetCoincidenceASCIIMask(G4bool);
  static void SetCoincidenceASCIIMask(G4std::vector<G4bool>);
  static G4std::vector<G4bool> GetCoincidenceASCIIMask();
  static G4bool GetCoincidenceASCIIMask(G4int index);

protected:
  static G4std::vector<G4bool> m_coincidenceASCIIMask;
  static G4bool                m_coincidenceASCIIMaskDefault;
};





typedef G4TDigiCollection<GateCoincidenceDigi> GateCoincidenceDigiCollection;

extern G4Allocator<GateCoincidenceDigi> GateCoincidenceDigiAllocator;





inline void* GateCoincidenceDigi::operator new(size_t)
{
  void* aDigi;
  aDigi = (void*) GateCoincidenceDigiAllocator.MallocSingle();
  return aDigi;
}





inline void GateCoincidenceDigi::operator delete(void* aDigi)
{
  GateCoincidenceDigiAllocator.FreeSingle((GateCoincidenceDigi*) aDigi);
}

#endif









