/*!
  \file GateToASCIIMessenger.hh
  
  $Log: GateToASCIIMessenger.hh,v $
  Revision 1.7  2004/09/16 13:59:36  gsantin
  FileSizeLimit and Coinc/Single masks

  Revision 1.6  2003/06/21 12:01:23  dstrul
  Output modules now handle multiple digi channels

  Revision 1.5  2002/11/24 15:10:41  gsantin
  new RecordVoxels, with disabling flag

  Revision 1.4  2002/10/21 15:23:29  dstrul
  Adaptations for DigiGate: simplified the use of the compilation options, added security for the case when the user tries to run DigiGate without the ROOT option

  Revision 1.3  2002/10/19 14:54:21  dstrul
  Adaptations for DigiGate: new messenger commands allow the user to set the name of the output files

  Revision 1.2  2002/08/11 15:33:24  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateToASCIIMessenger
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateToASCIIMessenger.hh,v 1.7 2004/09/16 13:59:36 gsantin Exp $
*/

#ifndef GateToASCIIMessenger_h
#define GateToASCIIMessenger_h 1
#ifdef G4ANALYSIS_USE_FILE

#include "GateOutputModuleMessenger.hh"
#include "g4std/vector"

#include "GateToASCII.hh"

class G4UIdirectory;
class G4UIcmdWithAString;
class G4UIcmdWithABool;
class G4UIcmdWithAnInteger;
class G4UIcmdWithADouble;
class G4UIcmdWithADoubleAndUnit;
class G4UIcmdWith3VectorAndUnit;
class G4UIcmdWithoutParameter;

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

class GateToASCIIMessenger: public GateOutputModuleMessenger
{
  public:
    GateToASCIIMessenger(GateToASCII* gateToASCII);
   ~GateToASCIIMessenger();
    
    void CreateNewOutputChannelCommand(GateToASCII::VOutputChannel* anOutputChannel);

    void SetNewValue(G4UIcommand*, G4String);

    G4bool IsAnOutputChannelCmd(G4UIcommand* command);
    void ExecuteOutputChannelCmd(G4UIcommand* command,G4String newValue);

  protected:
    GateToASCII*             			 m_gateToASCII;
    
    G4UIcmdWithoutParameter*          		 ResetCmd;
    G4UIcmdWithABool*                 		 OutFileHitsCmd;
    G4UIcmdWithABool*                 		 OutFileVoxelCmd;
    G4UIcmdWithAString*               		 SetFileNameCmd;

    G4std::vector<G4UIcmdWithABool*>  		 OutputChannelCmdList;
    G4std::vector<GateToASCII::VOutputChannel*>  m_outputChannelList;

    G4UIcommand* CoincidenceMaskCmd;
    G4int m_coincidenceMaskLength;

    G4UIcommand* SingleMaskCmd;
    G4int m_singleMaskLength;

    G4UIcmdWithAnInteger*                 	 SetOutFileSizeLimitCmd;

};

#endif
#endif
