/*!

  \file GateCoincidenceDigi.cc
  
  $Log: GateCoincidenceDigi.cc,v $
  Revision 1.12  2004/09/16 14:00:21  gsantin
  FileSizeLimit and Coinc/Single masks

  Revision 1.11  2004/02/13 15:42:22  sjan
  C.Groiselle Modifications about Compton OutPut Variables

  Revision 1.10  2003/06/03 12:21:52  sjan
  Compton Counting in the detector

  Revision 1.9  2002/11/22 10:59:36  dstrul
  Adaptations for compatibility with gcc3.1: added "G4std::" whenever needed

  Revision 1.8  2002/10/11 20:18:55  dstrul
  Removed the obsolete FFCPos fields

  Revision 1.7  2002/09/19 06:35:57  dstrul
  Removed all setw() to avoid incorrect characters on RedHat7.2

  Revision 1.6  2002/08/11 15:33:24  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateCoincidenceDigi
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateCoincidenceDigi.cc,v 1.12 2004/09/16 14:00:21 gsantin Exp $
*/

#include "GateCoincidenceDigi.hh"

#include "G4UnitsTable.hh"

#include "GatePulse.hh"
#include "GateCoincidencePulse.hh"

#include "g4std/iomanip"
#include "g4std/fstream"


G4std::vector<G4bool> GateCoincidenceDigi::m_coincidenceASCIIMask;
G4bool                GateCoincidenceDigi::m_coincidenceASCIIMaskDefault;

G4Allocator<GateCoincidenceDigi> GateCoincidenceDigiAllocator;



GateCoincidenceDigi::GateCoincidenceDigi()
{
  pulseVector[0] = GatePulse();
  pulseVector[1] = GatePulse();
}



GateCoincidenceDigi::GateCoincidenceDigi(GateCoincidencePulse* coincidencePulse)
{
  pulseVector[0] = (*coincidencePulse)[0];
  pulseVector[1] = (*coincidencePulse)[1];
}


GateCoincidenceDigi::GateCoincidenceDigi(const GateCoincidencePulse& coincidencePulse)
{
  pulseVector[0] = coincidencePulse[0];
  pulseVector[1] = coincidencePulse[1];
}


void GateCoincidenceDigi::Draw()
{;}





void GateCoincidenceDigi::Print()
{

  G4cout << this << G4endl;

}



G4std::ostream& operator<<(G4std::ostream& flux, GateCoincidenceDigi& digi)    
{
  flux    << "GateCoincidenceDigi("
	  << digi.pulseVector[0] << G4endl
	  << digi.pulseVector[1] << G4endl
      	  << ")" << G4endl;
    
  return flux;
}

G4std::ofstream& operator<<(G4std::ofstream& flux, GateCoincidenceDigi* digi)    
{
  GatePulse pulse;
  for (G4int iP=0; iP<2; iP++) {
    pulse = digi->GetPulse(iP);
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(0) ) flux << " " << G4std::setw(7) << pulse.GetRunID();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(1) ) flux << " " << G4std::setw(7) << pulse.GetEventID();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(2) ) flux << " " << G4std::setw(5) << pulse.GetSourceID();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(3) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetSourcePosition().x()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(4) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetSourcePosition().y()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(5) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetSourcePosition().z()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(6) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(23) << pulse.GetTime()/s;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(7) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetEnergy()/MeV;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(8) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetGlobalPos().x()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(9) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetGlobalPos().y()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(10) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetGlobalPos().z()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(11) ) flux << " " << G4std::setw(5) << pulse.GetOutputVolumeID();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(12) ) flux << " " << G4std::setw(5) << pulse.GetNPhantomCompton();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(13) ) flux << " " << G4std::setw(5) << pulse.GetNCrystalCompton();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(14) ) flux << " " << G4std::setw(5) << pulse.GetNPhantomRayleigh();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(15) ) flux << " " << G4std::setw(5) << pulse.GetNCrystalRayleigh();
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(16) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetScannerPos().z()/mm;
    if ( GateCoincidenceDigi::GetCoincidenceASCIIMask(17) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setprecision(3)  << pulse.GetScannerRotAngle()/deg;;
  }
  flux << G4endl;

  return flux;
}



void GateCoincidenceDigi::SetCoincidenceASCIIMask(G4bool newValue)
{
  m_coincidenceASCIIMaskDefault = newValue;
  for (G4int iMask=0; iMask<m_coincidenceASCIIMask.size(); iMask++) {
    m_coincidenceASCIIMask[iMask] = newValue;
  }
}


void GateCoincidenceDigi::SetCoincidenceASCIIMask(G4std::vector<G4bool> newMask)
{
  m_coincidenceASCIIMask = newMask;
}

G4bool GateCoincidenceDigi::GetCoincidenceASCIIMask(G4int index)
{
  G4bool mask = m_coincidenceASCIIMaskDefault;
  if ((index >=0 ) && (index < m_coincidenceASCIIMask.size())) mask = m_coincidenceASCIIMask[index];
  return mask;
}

G4std::vector<G4bool> GateCoincidenceDigi::GetCoincidenceASCIIMask()
{
  return m_coincidenceASCIIMask;
}
