/*!
  \file GateSingleDigi.cc
  
  $Log: GateSingleDigi.cc,v $
  Revision 1.12  2004/09/16 14:00:21  gsantin
  FileSizeLimit and Coinc/Single masks

  Revision 1.11  2004/02/13 15:43:17  sjan
  C.Groiselle Modifications about Compton OutPut Variables

  Revision 1.10  2003/06/03 12:22:16  sjan
  Compton Counting in the detector

  Revision 1.9  2002/11/29 17:26:29  dstrul
  Bug fix: corrected a bug introduced in r1.7, where the unit for z was stored in meters rather than in millimeters

  Revision 1.8  2002/11/22 10:59:55  dstrul
  Adaptations for compatibility with gcc3.1: added "G4std::" whenever needed

  Revision 1.7  2002/10/07 20:04:33  dstrul
  Added information in pulses and digis for the name of the last volume where a Compton interaction took place

  Revision 1.6  2002/08/11 15:33:25  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateSingleDigi
  \brief By Giovanni.Santin@cern.ch
  \brief $Id: GateSingleDigi.cc,v 1.12 2004/09/16 14:00:21 gsantin Exp $
*/

#include "GateSingleDigi.hh"

#include "G4UnitsTable.hh"

#include "GatePulse.hh"

#include "g4std/iomanip"


G4std::vector<G4bool> GateSingleDigi::m_singleASCIIMask;
G4bool                GateSingleDigi::m_singleASCIIMaskDefault;

G4Allocator<GateSingleDigi> GateSingleDigiAllocator;




GateSingleDigi::GateSingleDigi()
  : m_pulse()
{
}



GateSingleDigi::GateSingleDigi(GatePulse* pulse)
  : m_pulse(*pulse)
{
}


GateSingleDigi::GateSingleDigi(const GatePulse& pulse)
  : m_pulse(pulse)
{
}




void GateSingleDigi::Draw()
{;}





void GateSingleDigi::Print()
{

  G4cout << this << G4endl;

}



G4std::ostream& operator<<(G4std::ostream& flux, const GateSingleDigi& digi)    
{
  flux    << "GateSingleDigi(" << G4endl
	  << digi.m_pulse
      	  << ")" << G4endl;
    
  return flux;
}

G4std::ofstream& operator<<(G4std::ofstream& flux, GateSingleDigi* digi)    
{
  if ( GateSingleDigi::GetSingleASCIIMask(0) ) flux << " " << G4std::setw(7) << digi->GetRunID();
  if ( GateSingleDigi::GetSingleASCIIMask(1) ) flux << " " << G4std::setw(7) << digi->GetEventID();
  if ( GateSingleDigi::GetSingleASCIIMask(2) ) flux << " " << G4std::setw(5) << digi->GetSourceID();
  if ( GateSingleDigi::GetSingleASCIIMask(3) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3) << digi->GetSourcePosition().x()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(4) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3) << digi->GetSourcePosition().y()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(5) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3) << digi->GetSourcePosition().z()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(6) ) flux << " " << G4std::setw(5) << digi->GetOutputVolumeID();
  if ( GateSingleDigi::GetSingleASCIIMask(7) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(30) << G4std::setprecision(23) << digi->GetTime()/s;
  if ( GateSingleDigi::GetSingleASCIIMask(8) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << digi->GetEnergy()/MeV;
  if ( GateSingleDigi::GetSingleASCIIMask(9) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << digi->GetGlobalPos().x()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(10) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << digi->GetGlobalPos().y()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(11) ) flux << " " << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << digi->GetGlobalPos().z()/mm;
  if ( GateSingleDigi::GetSingleASCIIMask(12) ) flux << " " << G4std::setw(4) << digi->GetNPhantomCompton();
  if ( GateSingleDigi::GetSingleASCIIMask(13) ) flux << " " << G4std::setw(4) << digi->GetNCrystalCompton();
  if ( GateSingleDigi::GetSingleASCIIMask(14) ) flux << " " << G4std::setw(4) << digi->GetNPhantomRayleigh();
  if ( GateSingleDigi::GetSingleASCIIMask(15) ) flux << " " << G4std::setw(4) << digi->GetNCrystalRayleigh();
  if ( GateSingleDigi::GetSingleASCIIMask(16) ) flux << " " << digi->GetComptonVolumeName();
  if ( GateSingleDigi::GetSingleASCIIMask(17) ) flux << " " << digi->GetRayleighVolumeName();
  flux << G4endl;	
    
  return flux;
}



void GateSingleDigi::SetSingleASCIIMask(G4bool newValue)
{
  m_singleASCIIMaskDefault = newValue;
  for (G4int iMask=0; iMask<m_singleASCIIMask.size(); iMask++) {
    m_singleASCIIMask[iMask] = newValue;
  }
}


void GateSingleDigi::SetSingleASCIIMask(G4std::vector<G4bool> newMask)
{
  m_singleASCIIMask = newMask;
}

G4bool GateSingleDigi::GetSingleASCIIMask(G4int index)
{
  G4bool mask = m_singleASCIIMaskDefault;
  if ((index >=0 ) && (index < m_singleASCIIMask.size())) mask = m_singleASCIIMask[index];
  return mask;
}

G4std::vector<G4bool> GateSingleDigi::GetSingleASCIIMask()
{
  return m_singleASCIIMask;
}








