/*!
  \file GateToASCII.cc
  
  $Log: GateToASCII.cc,v $
  Revision 1.25  2004/09/16 14:00:21  gsantin
  FileSizeLimit and Coinc/Single masks

  Revision 1.24  2004/04/21 09:35:42  fmayet
  test

  Revision 1.23  2003/11/12 11:10:22  dstrul
  Bug fix: corrected problem with setting of output file name for Singles and Coincidences files

  Revision 1.22  2003/07/02 22:18:29  dstrul
  Digitizer re-engineering

  Revision 1.21  2003/06/24 17:31:31  gsantin
  syntax bool? char : G4String --> G4String(char)

  Revision 1.20  2003/06/21 12:01:23  dstrul
  Output modules now handle multiple digi channels

  Revision 1.19  2003/06/17 12:00:30  dstrul
  Replaced inclusions of iomanip.h and fstream.h with inclusion of g4std/iomanip and g4std/fstream to suppress compilation warning; Replaced call to std::setw() with call to G4std::setw() as needed

  Revision 1.18  2002/11/24 15:10:01  gsantin
  new RecordVoxels, with disabling flag

  Revision 1.17  2002/11/22 10:57:48  dstrul
  Adaptations for compatibility with gcc3.1: added "G4std::" whenever needed

  Revision 1.16  2002/10/21 15:23:29  dstrul
  Adaptations for DigiGate: simplified the use of the compilation options, added security for the case when the user tries to run DigiGate without the ROOT option

  Revision 1.15  2002/10/20 13:14:04  dstrul
  GateCrystalHit now provides a method GoodForAnalysis() to tell whether a hit should be analysed then stored into the output hit files

  Revision 1.14  2002/10/20 12:42:30  dstrul
  Adaptations for DigiGate: transportation hits with non-zero energy deposition now are analysed and written to the output hit files

  Revision 1.13  2002/10/20 11:28:18  dstrul
  Adaptations for DigiGate: GateToASCII and GateToDigi now accept to process the hits even when there is no trajectory container

  Revision 1.12  2002/10/19 14:53:39  dstrul
  Adaptations for Digigate: the opening and closinf of the output files has been transfered into the new methods RecordBeginOfAcquisition() and RecordEndOfAcquisition(); the name of the output file can be set by the user

  Revision 1.11  2002/10/19 13:15:51  dstrul
  Adaptations for DigiGate: the hit data are no longer written when in off-line digitation mode

  Revision 1.10  2002/10/19 11:49:12  dstrul
  Adaptations for DigiGate: all requests for hit or digi collection are now sent to the output-mgr

  Revision 1.9  2002/10/19 10:50:11  dstrul
  Adaptations for DigiGate: the retrieving of the hits and digis collections is now handled by methods provided by the base-class GateVOutputModule

  Revision 1.8  2002/08/11 15:33:25  dstrul
  Cosmetic cleanup: standardized file comments for cleaner doxygen output


  \brief Class GateToASCII
  \brief By lazaro@clermont.in2p3.fr
  \brief $Id: GateToASCII.cc,v 1.25 2004/09/16 14:00:21 gsantin Exp $
*/
#ifdef G4ANALYSIS_USE_FILE

#include "GateToASCII.hh"
#include "GateToASCIIMessenger.hh"
#include "GateVGeometryVoxelStore.hh"

#include "globals.hh"

#include "G4Run.hh"
#include "G4Step.hh"
#include "G4Event.hh"
#include "GateCrystalHit.hh"
#include "GatePhantomHit.hh"
#include "G4VHitsCollection.hh"

#include "G4TrajectoryContainer.hh"

#include "G4VProcess.hh"
#include "RecorderBase.hh"
#include "G4ios.hh"
#include "g4std/iomanip"
#include "G4UImanager.hh"
#include "G4RunManager.hh"
#include "GatePrimaryGeneratorAction.hh"

#include "G4ParticleDefinition.hh"
#include "G4Positron.hh"
#include "G4GenericIon.hh"

#include "GateDigitizer.hh"
#include "GateSingleDigi.hh"
#include "GateCoincidenceDigi.hh"

#include "GateSourceMgr.hh"
#include "GateOutputMgr.hh"
#include "G4DigiManager.hh"

#include <iostream.h>
#include <fstream.h>

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

GateToASCII::GateToASCII(const G4String& name, GateOutputMgr* outputMgr,DigiMode digiMode) 
  : GateVOutputModule(name,outputMgr,digiMode)
  ,m_outFileRunsFlag(digiMode==kruntimeMode)
  ,m_outFileHitsFlag(digiMode==kruntimeMode)
  ,m_outFileVoxelFlag(true)
  ,m_fileName("gate")
{
  if (digiMode==kofflineMode)
    m_fileName="digigate";

  m_asciiMessenger = new GateToASCIIMessenger(this);

  m_verboseLevel =0;

  GateCoincidenceDigi::SetCoincidenceASCIIMask(1);
  GateSingleDigi::SetSingleASCIIMask(1);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo...

GateToASCII::~GateToASCII() 
{
  for (size_t i=0; i<m_outputChannelList.size() ; ++i )
    delete m_outputChannelList[i];

  delete m_asciiMessenger;

  if (m_verboseLevel > 0) G4cout << "GateToASCII deleting..." << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


void GateToASCII::RecordBeginOfAcquisition()
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordBeginOfAcquisition" << G4endl;

  if (m_verboseLevel > 0) G4cout << "Opening the ASCII output files...";
  if (m_outFileRunsFlag)
    m_outFileRun.open((m_fileName+"Run.dat").c_str(),G4std::ios::out);
  if (m_outFileHitsFlag)
    m_outFileHits.open((m_fileName+"Hits.dat").c_str(),G4std::ios::out);

  for (size_t i=0; i<m_outputChannelList.size() ; ++i )
    m_outputChannelList[i]->Open(m_fileName);

  if (m_verboseLevel > 0) G4cout << " ... ASCII output files opened" << G4endl; 
}

 


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....


void GateToASCII::RecordEndOfAcquisition()
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordEndOfAcquisition" << G4endl;
  // Close the file with the hits information
  if (m_outFileRunsFlag)
    m_outFileRun.close();
  if (m_outFileHitsFlag)
    m_outFileHits.close();

  for (size_t i=0; i<m_outputChannelList.size() ; ++i )
       m_outputChannelList[i]->Close();

}



//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordBeginOfRun(const G4Run * r)
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordBeginOfRun" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordEndOfRun(const G4Run * r)
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordEndOfRun" << G4endl;
  if (m_outFileRunsFlag) {
    G4int nEvent = ((GatePrimaryGeneratorAction*)G4RunManager::GetRunManager()->
		    GetUserPrimaryGeneratorAction())->GetEventNumber();
    if (m_verboseLevel > 0) G4cout 
      << "GateToASCII::RecordEndOfRun: Events in the past run: " << nEvent << G4endl;
    m_outFileRun
      << " " << G4std::setw(9) << nEvent
      << G4endl;
  }

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordBeginOfEvent(const G4Event* event)
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordBeginOfEvent" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordEndOfEvent(const G4Event* event)
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordEndOfEvent" << G4endl;

  if (m_outFileHitsFlag) {

    GateCrystalHitsCollection* CHC = GetOutputMgr()->GetCrystalHitCollection();

    G4int NbHits = 0;

    if (CHC) {    

      // Hits loop

      NbHits = CHC->entries();
      for (G4int iHit=0;iHit<NbHits;iHit++) {
	G4String processName = (*CHC)[iHit]->GetProcess();
	G4int PDGEncoding  = (*CHC)[iHit]->GetPDGEncoding();
	if (m_verboseLevel > 2) G4cout 
	  << "GateToASCII::RecordEndOfEvent : CrystalHitsCollection: processName : <" << processName 
	  << ">    Particls PDG code : " << PDGEncoding << G4endl;
	if ((*CHC)[iHit]->GoodForAnalysis()) {
	  if (m_outFileHitsFlag) m_outFileHits << (*CHC)[iHit];
	}
      }
      
    }
    else{
      if (m_verboseLevel>0) G4cout << "GateToASCII::RecordHits : GateCrystalHitCollection not found" << G4endl;
    } 
  }


  RecordDigitizer(event);

}
//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordDigitizer(const G4Event* event) 
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordDigitizer" << G4endl;

  for (size_t i=0; i<m_outputChannelList.size() ; ++i )
    m_outputChannelList[i]->RecordDigitizer();
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordStep(const G4Step* aStep)
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::RecordStep" << G4endl;
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RecordVoxels(GateVGeometryVoxelStore* voxelStore)
{
  if (m_verboseLevel > 2)
    G4cout << "[GateToASCII::RecordVoxels]" << G4endl;

  // protect against huge ASCII files in case of nx,ny,nz ~O(100)
  if (!m_outFileVoxelFlag) return;

  G4std::ofstream  voxelFile; 	      	    //!< Output stream for the voxel density map
  // Open the header file
  G4String voxelFileName = "voxels.dat";
  voxelFile.open(voxelFileName.c_str(),G4std::ios::out | G4std::ios::trunc);
  if (!(voxelFile.is_open()))
      	G4Exception("[GateToASCII::RecordVoxels]:\n"
      	      	    "Could not open the voxel file '" + voxelFileName);

  // Write the header: number of voxels, voxel dimensions
  G4int nx = voxelStore->GetVoxelNx();
  G4int ny = voxelStore->GetVoxelNy();
  G4int nz = voxelStore->GetVoxelNz();
  G4ThreeVector voxelSize = voxelStore->GetVoxelSize();
  G4double dx = voxelSize.x();
  G4double dy = voxelSize.y();
  G4double dz = voxelSize.z();

  voxelFile << " " << nx << " " << ny << " " << nz << G4endl;
  voxelFile << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << dx/mm; 
  voxelFile << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << dy/mm; 
  voxelFile << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << dz/mm; 
  voxelFile << G4endl;

  // Write the content of the voxel matrix
  for (G4int iz=0; iz<nz; iz++) {
    for (G4int iy=0; iy<ny; iy++) {
      for (G4int ix=0; ix<nx; ix++) {
	G4double density = voxelStore->GetVoxelMaterial(ix,iy,iz)->GetDensity()/(gram/cm3);
	//	G4cout << "Material: " << voxelStore->GetVoxelMaterial(ix,iy,iz)->GetName() << "  density: " << density << G4endl;
	voxelFile << G4std::resetiosflags(G4std::ios::floatfield) << G4std::setiosflags(G4std::ios::scientific) << G4std::setw(10) << G4std::setprecision(3)  << density;
      }
      // line break for each voxel line
      voxelFile << G4endl;
    }
  }

  voxelFile.close();

}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::Reset()
{
  if (m_verboseLevel > 2)
    G4cout << "GateToASCII::Reset" << G4endl;
}


//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RegisterNewSingleDigiCollection(const G4String& aCollectionName,G4bool outputFlag)
{
  SingleOutputChannel* singleOutputChannel = 
    new SingleOutputChannel(aCollectionName,outputFlag);
  m_outputChannelList.push_back(singleOutputChannel);
  m_asciiMessenger->CreateNewOutputChannelCommand(singleOutputChannel);
}

//....oooOO0OOooo........oooOO0OOooo........oooOO0OOooo........oooOO0OOooo....

void GateToASCII::RegisterNewCoincidenceDigiCollection(const G4String& aCollectionName,G4bool outputFlag)
{
  CoincidenceOutputChannel* coincOutputChannel = 
    new CoincidenceOutputChannel(aCollectionName,outputFlag);
  m_outputChannelList.push_back(coincOutputChannel);
  m_asciiMessenger->CreateNewOutputChannelCommand(coincOutputChannel);
}

long GateToASCII::VOutputChannel::m_outputFileSizeLimit = 2000000000;

void GateToASCII::VOutputChannel::Open(const G4String& aFileBaseName)
{
  // if it's not the first file with the same name, add a suffix like _01 to the file name, before .dat
  if ((m_fileCounter > 0) && (m_fileBaseName != aFileBaseName)) {
    m_fileCounter = 0;
  }

  G4String fileCounterSuffix;
  if (m_fileCounter > 0) {
    G4String fileCounterString;
    char buffer [10];
    sprintf(buffer,"%d",m_fileCounter);
    fileCounterString = buffer;
    fileCounterSuffix = G4String("_") + fileCounterString;
  } else {
    fileCounterSuffix = G4String("");
  }
  G4String fileName = aFileBaseName + m_collectionName + fileCounterSuffix + ".dat";
  if (m_outputFlag) {
    m_outputFile.open(fileName,G4std::ios::out);
    m_outputFile.seekp (0, ios::beg);
    m_outputFileBegin = m_outputFile.tellp();
  }
  m_fileBaseName = aFileBaseName;
  m_fileCounter++;
}
      

void GateToASCII::VOutputChannel::Close()
{
  if (m_outputFlag)
    m_outputFile.close();
}
      
G4bool GateToASCII::VOutputChannel::ExceedsSize()
{
  // from http://www.cplusplus.com/doc/tutorial/tut6-1.html
  long outputFileEnd;
  m_outputFile.seekp (0, ios::end);
  outputFileEnd = m_outputFile.tellp();
  long size = outputFileEnd - m_outputFileBegin; // in bytes
//   G4cout << "[GateToASCII::VOutputChannel::ExceedsSize]" 
// 	 << " collectionID: " << m_collectionID 
// 	 << " file limit: " << m_outputFileSizeLimit
// 	 << " file size: " << size << G4endl;
  return (size > m_outputFileSizeLimit);
}
      

GateToASCII::SingleOutputChannel::SingleOutputChannel(  const G4String& aCollectionName,
							G4bool outputFlag)
  : GateToASCII::VOutputChannel( aCollectionName, outputFlag )
{
}


void GateToASCII::SingleOutputChannel::RecordDigitizer() 
{
  G4DigiManager * fDM = G4DigiManager::GetDMpointer();
  if (m_collectionID<0)
    m_collectionID = fDM->GetDigiCollectionID(m_collectionName);      
  const GateSingleDigiCollection * SDC = 
    (GateSingleDigiCollection*) (fDM->GetDigiCollection( m_collectionID ));

  if (!SDC) {
    if (m_verboseLevel>0) G4cout << "[GateToASCII::SingleOutputChannel::RecordDigitizer]: "
    				 << "digi collection '" << m_collectionName <<"' not found" << G4endl;
  } else {
    // Digi loop
    if (m_verboseLevel>0) G4cout << "[GateToASCII::SingleOutputChannel::RecordDigitizer]: Totals digits: "
				 << SDC->entries() << G4endl;
    if (m_outputFlag) {
      G4int n_digi =  SDC->entries();
      for (G4int iDigi=0;iDigi<n_digi;iDigi++) {
	if (m_outputFileSizeLimit > 10000) { // to protect against the creation of too many files by mistake
	  if (ExceedsSize()) {
	    Close();
	    Open(m_fileBaseName);
	  }
	}
        m_outputFile << (*SDC)[iDigi];
      }
    }    

  }

}

GateToASCII::CoincidenceOutputChannel::CoincidenceOutputChannel(const G4String& aCollectionName,
								G4bool outputFlag)
  : GateToASCII::VOutputChannel( aCollectionName, outputFlag)
{
}


void GateToASCII::CoincidenceOutputChannel::RecordDigitizer() 
{
  G4DigiManager * fDM = G4DigiManager::GetDMpointer();
  if (m_collectionID<0)
    m_collectionID = fDM->GetDigiCollectionID(m_collectionName);      
  GateCoincidenceDigiCollection * CDC = 
    (GateCoincidenceDigiCollection*) (fDM->GetDigiCollection( m_collectionID ));

  if (!CDC) {
    if (m_verboseLevel>0) G4cout << "[GateToASCII::CoincidenceOutputChannel::RecordDigitizer]: "
    				 << "digi collection '" << m_collectionName <<"' not found" << G4endl;
  } else {
    // Digi loop
    if (m_verboseLevel>0) G4cout << "[GateToASCII::CoincidenceOutputChannel::RecordDigitizer]: Totals digits: "
				 << CDC->entries() << G4endl;
    
    if (m_outputFlag) {
      G4int n_digi =  CDC->entries();
      for (G4int iDigi=0;iDigi<n_digi;iDigi++) {
	if (m_outputFileSizeLimit > 10000) { // to protect against the creation of too many files by mistake
	  if (ExceedsSize()) {
	    Close();
	    Open(m_fileBaseName);
	  }
	}
	m_outputFile << (*CDC)[iDigi];
      }
    }
  }

  
}

#endif







